/* IMPORT */
import { isWeakReferrable } from './utils.js';
/* MAIN */
class MildMap {
    /* VARIABLES */
    #strong;
    #weak;
    /* CONSTRUCTOR */
    constructor(entries) {
        /* VARIABLES */
        this.#strong = new Map();
        this.#weak = new WeakMap();
        if (entries) {
            for (const [key, value] of entries) {
                this.set(key, value);
            }
        }
    }
    /* API */
    delete(key) {
        if (isWeakReferrable(key)) {
            return this.#weak.delete(key);
        }
        else {
            return this.#strong.delete(key);
        }
    }
    get(key) {
        if (isWeakReferrable(key)) {
            return this.#weak.get(key);
        }
        else {
            return this.#strong.get(key);
        }
    }
    has(key) {
        if (isWeakReferrable(key)) {
            return this.#weak.has(key);
        }
        else {
            return this.#strong.has(key);
        }
    }
    set(key, value) {
        if (isWeakReferrable(key)) {
            this.#weak.set(key, value);
        }
        else {
            this.#strong.set(key, value);
        }
        return this;
    }
}
/* EXPORT */
export default MildMap;
